@@ -28,7 +28,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
+#include <math.h>
+
 #include "conversions.h"
+#include "rel_assert.h"
 
 enum str2args_parse_state {
     PARSE_STATE_IN_SPACE,
@@ -139,10 +142,30 @@ double str2double(const char *str, double min, double max, bool *ok)
     return value;
 }
 
-unsigned int str2uint_suffix(const char *str,
-                             unsigned int min, unsigned int max,
-                             const struct numeric_suffix suffixes[],
-                             size_t num_suffixes, bool *ok)
+/* MSVC workarounds */
+
+#if _MSC_VER
+    /* This appears to be missing <= 2012 */
+#   ifndef INFINITY
+#       define INFINITY (_HUGE * _HUGE)
+#   endif
+
+   /* As of MSVC 2013, INFINITY appears to be available, but
+    * the compiler emits a warning for every usage, despite it
+    * causing an overflow by design.
+    *  https://msdn.microsoft.com/en-us/library/cwt7tyxx.aspx
+    *
+    * Oddly, we see warning 4056, despite math.h noting that 4756
+    * will be induced (with the same description).
+    */
+#   pragma warning (push)
+#   pragma warning (disable:4056)
+#endif
+
+double str2dbl_suffix(const char *str,
+                      double min, double max,
+                      const struct numeric_suffix suffixes[],
+                      size_t num_suffixes, bool *ok)
 {
     double value;
     char *endptr;
@@ -170,6 +193,15 @@ unsigned int str2uint_suffix(const char *str,
         }
     }
 
+    /* Test for overflow */
+    if (value == INFINITY || value == -INFINITY) {
+        if (ok) {
+            *ok = false;
+        }
+
+        return 0;
+    }
+
     /* Check that the resulting value is in bounds */
     if (value > max || value < min) {
         if (ok) {
@@ -183,10 +215,30 @@ unsigned int str2uint_suffix(const char *str,
         *ok = true;
     }
 
-    /* Truncate the floating point value to an integer and return it */
-    return (unsigned int)value;
+    return value;
+}
+#if _MSC_VER
+#   pragma warning(pop)
+#endif
+
+unsigned int str2uint_suffix(const char *str,
+                             unsigned int min, unsigned int max,
+                             const struct numeric_suffix suffixes[],
+                             size_t num_suffixes, bool *ok)
+{
+    return (unsigned int) str2dbl_suffix(str, min, max,
+                                         suffixes, num_suffixes, ok);
 }
 
+uint64_t str2uint64_suffix(const char *str,
+                           uint64_t min, uint64_t max,
+                           const struct numeric_suffix suffixes[],
+                           size_t num_suffixes, bool *ok)
+{
+    /* FIXME: Potential loss of precision on min/max here */
+    return (uint64_t) str2dbl_suffix(str, (double) min, (double) max,
+                                     suffixes, num_suffixes, ok);
+}
 int str2version(const char *str, struct bladerf_version *version)
 {
     unsigned long tmp;
@@ -329,6 +381,103 @@ const char * module2str(bladerf_module m)
     }
 }
 
+bladerf_module str2module(const char *str)
+{
+    if (!strcasecmp(str, "RX")) {
+        return BLADERF_MODULE_RX;
+    } else if (!strcasecmp(str, "TX")) {
+        return BLADERF_MODULE_TX;
+    } else {
+        return BLADERF_MODULE_INVALID;
+    }
+}
+
+const char * trigger2str(bladerf_trigger_signal trigger)
+{
+    switch (trigger) {
+        case BLADERF_TRIGGER_J71_4:
+            return "J71-4";
+
+        case BLADERF_TRIGGER_USER_0:
+            return "User-0";
+
+        case BLADERF_TRIGGER_USER_1:
+            return "User-1";
+
+        case BLADERF_TRIGGER_USER_2:
+            return "User-2";
+
+        case BLADERF_TRIGGER_USER_3:
+            return "User-3";
+
+        case BLADERF_TRIGGER_USER_4:
+            return "User-4";
+
+        case BLADERF_TRIGGER_USER_5:
+            return "User-5";
+
+        case BLADERF_TRIGGER_USER_6:
+            return "User-6";
+
+        case BLADERF_TRIGGER_USER_7:
+            return "User-7";
+
+        default:
+            return "Unknown";
+    }
+}
+
+bladerf_trigger_signal str2trigger(const char *str)
+{
+    if (!strcasecmp("J71-4", str)) {
+        return BLADERF_TRIGGER_J71_4;
+    } else if (!strcasecmp("User-0", str)) {
+        return BLADERF_TRIGGER_USER_0;
+    } else if (!strcasecmp("User-1", str)) {
+        return BLADERF_TRIGGER_USER_1;
+    } else if (!strcasecmp("User-2", str)) {
+        return BLADERF_TRIGGER_USER_2;
+    } else if (!strcasecmp("User-3", str)) {
+        return BLADERF_TRIGGER_USER_3;
+    } else if (!strcasecmp("User-4", str)) {
+        return BLADERF_TRIGGER_USER_4;
+    } else if (!strcasecmp("User-5", str)) {
+        return BLADERF_TRIGGER_USER_5;
+    } else if (!strcasecmp("User-6", str)) {
+        return BLADERF_TRIGGER_USER_6;
+    } else if (!strcasecmp("User-7", str)) {
+        return BLADERF_TRIGGER_USER_7;
+    } else {
+        return BLADERF_TRIGGER_INVALID;
+    }
+}
+
+const char *triggerrole2str(bladerf_trigger_role role)
+{
+    switch (role) {
+        case BLADERF_TRIGGER_ROLE_MASTER:
+            return "Master";
+        case BLADERF_TRIGGER_ROLE_SLAVE:
+            return "Slave";
+        case BLADERF_TRIGGER_ROLE_DISABLED:
+            return "Disabled";
+        default:
+            return "Unknown";
+    }
+}
+
+bladerf_trigger_role str2triggerrole(const char *str) {
+    if (!strcasecmp("Master", str)) {
+        return BLADERF_TRIGGER_ROLE_MASTER;
+    } else if (!strcasecmp("Slave", str)) {
+        return BLADERF_TRIGGER_ROLE_SLAVE;
+    } else if (!strcasecmp("Disabled", str) || !strcasecmp("Off", str)) {
+        return BLADERF_TRIGGER_ROLE_DISABLED;
+    } else {
+        return BLADERF_TRIGGER_ROLE_INVALID;
+    }
+}
+
 int str2loopback(const char *str, bladerf_loopback *loopback)
 {
     int status = 0;
@@ -572,3 +721,83 @@ void float_to_sc16q11(const float *in, int16_t *out, unsigned int n)
         out[i+1] = (int16_t) (in[i+1] * 2048.0f);
     }
 }
+
+bladerf_cal_module str_to_bladerf_cal_module(const char *str)
+{
+    bladerf_cal_module module = BLADERF_DC_CAL_INVALID;
+
+    if (!strcasecmp(str, "lpf_tuning") || !strcasecmp(str, "lpftuning") ||
+        !strcasecmp(str, "tuning")) {
+        module = BLADERF_DC_CAL_LPF_TUNING;
+    } else if (!strcasecmp(str, "tx_lpf")  || !strcasecmp(str, "txlpf")) {
+        module = BLADERF_DC_CAL_TX_LPF;
+    } else if (!strcasecmp(str, "rx_lpf")  || !strcasecmp(str, "rxlpf")) {
+        module = BLADERF_DC_CAL_RX_LPF;
+    } else if (!strcasecmp(str, "rx_vga2") || !strcasecmp(str, "rxvga2")) {
+        module = BLADERF_DC_CAL_RXVGA2;
+    }
+
+    return module;
+}
+
+int str2bool(const char *str, bool *val)
+{
+    static const char *t_vals[] = { "true",  "t", "enable",  "en",  "e", "on",  "1" };
+    static const char *f_vals[] = { "false", "f", "disable", "dis", "d", "off", "0" };
+
+    static const size_t t_vals_len = sizeof(t_vals) / sizeof(t_vals[0]);
+    static const size_t f_vals_len = sizeof(f_vals) / sizeof(f_vals[0]);
+
+    size_t i;
+
+    assert(t_vals_len == f_vals_len);
+
+    for (i = 0; i < sizeof(t_vals) / sizeof(t_vals[0]); i++) {
+        if (!strcasecmp(str, t_vals[i])) {
+            *val = true;
+            return 0;
+        } else if (!strcasecmp(str, f_vals[i])) {
+            *val = false;
+            return 0;
+        }
+    }
+
+    /* No match */
+    *val = false;
+    return -1;
+}
+
+const char * smb_mode_to_str(bladerf_smb_mode mode)
+{
+    switch (mode) {
+        case BLADERF_SMB_MODE_DISABLED:
+            return "Disabled";
+
+        case BLADERF_SMB_MODE_OUTPUT:
+            return "Output";
+
+        case BLADERF_SMB_MODE_INPUT:
+            return "Input";
+
+        case BLADERF_SMB_MODE_UNAVAILBLE:
+            return "Unavailable";
+
+        default:
+            return "Unknown";
+    }
+};
+
+bladerf_smb_mode str_to_smb_mode(const char *str)
+{
+    if (!strcasecmp(str, "disabled") || !strcasecmp(str, "off")) {
+        return BLADERF_SMB_MODE_DISABLED;
+    } else if (!strcasecmp(str, "output")) {
+        return BLADERF_SMB_MODE_OUTPUT;
+    } else if (!strcasecmp(str, "input")) {
+        return BLADERF_SMB_MODE_INPUT;
+    } else if (!strcasecmp(str, "unavailable")) {
+        return BLADERF_SMB_MODE_UNAVAILBLE;
+    } else {
+        return BLADERF_SMB_MODE_INVALID;
+    }
+}
