@@ -196,7 +196,7 @@ int nios_8x32_read(struct bladerf *dev, uint8_t id,
     uint8_t buf[NIOS_PKT_LEN];
     bool success;
 
-    nios_pkt_8x32_pack(buf, id, false, 0, 0);
+    nios_pkt_8x32_pack(buf, id, false, addr, 0);
 
     status = nios_access(dev, buf);
     if (status != 0) {
@@ -382,7 +382,7 @@ int nios_si5338_read(struct bladerf *dev, uint8_t addr, uint8_t *data)
 
     if (status == 0) {
         log_verbose("%s: Read 0x%02x from addr 0x%02x\n",
-                    __FUNCTION__, addr, *data);
+                    __FUNCTION__, *data, addr);
     }
 
     return status;
@@ -394,7 +394,7 @@ int nios_si5338_write(struct bladerf *dev, uint8_t addr, uint8_t data)
 
     if (status == 0) {
         log_verbose("%s: Wrote 0x%02x to addr 0x%02x\n",
-                    __FUNCTION__, addr, data);
+                    __FUNCTION__, data, addr);
     }
 
     return status;
@@ -406,7 +406,7 @@ int nios_lms6_read(struct bladerf *dev, uint8_t addr, uint8_t *data)
 
     if (status == 0) {
         log_verbose("%s: Read 0x%02x from addr 0x%02x\n",
-                    __FUNCTION__, addr, *data);
+                    __FUNCTION__, *data, addr);
     }
 
     return status;
@@ -418,7 +418,7 @@ int nios_lms6_write(struct bladerf *dev, uint8_t addr, uint8_t data)
 
     if (status == 0) {
         log_verbose("%s: Wrote 0x%02x to addr 0x%02x\n",
-                    __FUNCTION__, addr, data);
+                    __FUNCTION__, data, addr);
     }
 
     return status;
@@ -464,6 +464,65 @@ int nios_vctcxo_trim_dac_read(struct bladerf *dev, uint16_t *value)
     return status;
 }
 
+int nios_set_vctcxo_tamer_mode(struct bladerf *dev,
+                               bladerf_vctcxo_tamer_mode mode)
+{
+    int status;
+
+    if (!have_cap(dev, BLADERF_CAP_VCTCXO_TAMING_MODE)) {
+        log_debug("FPGA %s does not support VCTCXO taming via an input source\n",
+                  dev->fpga_version.describe);
+
+        return BLADERF_ERR_UNSUPPORTED;
+    }
+
+
+    status = nios_8x8_write(dev,
+                            NIOS_PKT_8x8_TARGET_VCTCXO_TAMER, 0xff,
+                            (uint8_t) mode);
+
+    if (status == 0) {
+        log_verbose("%s: Wrote mode=0x%02x\n", __FUNCTION__, mode);
+    }
+
+    return status;
+}
+
+int nios_get_vctcxo_tamer_mode(struct bladerf *dev,
+                               bladerf_vctcxo_tamer_mode *mode)
+{
+    int status;
+    uint8_t tmp;
+
+    *mode = BLADERF_VCTCXO_TAMER_INVALID;
+
+    if (!have_cap(dev, BLADERF_CAP_VCTCXO_TAMING_MODE)) {
+        log_debug("FPGA %s does not support VCTCXO taming via an input source\n",
+                  dev->fpga_version.describe);
+
+        return BLADERF_ERR_UNSUPPORTED;
+    }
+
+    status = nios_8x8_read(dev, NIOS_PKT_8x8_TARGET_VCTCXO_TAMER, 0xff, &tmp);
+    if (status == 0) {
+        log_verbose("%s: Read mode=0x%02x\n", __FUNCTION__, tmp);
+
+        switch ((bladerf_vctcxo_tamer_mode) tmp) {
+            case BLADERF_VCTCXO_TAMER_DISABLED:
+            case BLADERF_VCTCXO_TAMER_1_PPS:
+            case BLADERF_VCTCXO_TAMER_10_MHZ:
+                *mode = (bladerf_vctcxo_tamer_mode) tmp;
+                break;
+
+            default:
+                status = BLADERF_ERR_UNEXPECTED;
+        }
+    }
+
+    return status;
+}
+
+
 int nios_get_iq_gain_correction(struct bladerf *dev, bladerf_module module,
                                 int16_t *value)
 {
@@ -598,28 +657,26 @@ int nios_xb200_synth_write(struct bladerf *dev, uint32_t value)
     return status;
 }
 
-/* The update format for GPIO (and the GPIO direction register) accesses allows
- * a bitmask to be provided.  For this interface, we just want to read all bits
- * of the GPIO bank. */
 int nios_expansion_gpio_read(struct bladerf *dev, uint32_t *val)
 {
     int status = nios_32x32_masked_read(dev, NIOS_PKT_32x32_TARGET_EXP,
                                         0xffffffff, val);
 
     if (status == 0) {
-        log_verbose("%s: Read 0x08x\n", __FUNCTION__, *val);
+        log_verbose("%s: Read 0x%08x\n", __FUNCTION__, *val);
     }
 
     return status;
 }
 
-int nios_expansion_gpio_write(struct bladerf *dev, uint32_t val)
+int nios_expansion_gpio_write(struct bladerf *dev, uint32_t mask, uint32_t val)
 {
     int status = nios_32x32_masked_write(dev, NIOS_PKT_32x32_TARGET_EXP,
-                                         0xffffffff, val);
+                                         mask, val);
 
     if (status == 0) {
-        log_verbose("%s: Wrote 0x08x\n", __FUNCTION__, val);
+        log_verbose("%s: Wrote 0x%08x (with mask 0x%08x)\n",
+                    __FUNCTION__, val, mask);
     }
 
     return status;
@@ -631,19 +688,21 @@ int nios_expansion_gpio_dir_read(struct bladerf *dev, uint32_t *val)
                                         0xffffffff, val);
 
     if (status == 0) {
-        log_verbose("%s: Read 0x08x\n", __FUNCTION__, *val);
+        log_verbose("%s: Read 0x%08x\n", __FUNCTION__, *val);
     }
 
     return status;
 }
 
-int nios_expansion_gpio_dir_write(struct bladerf *dev, uint32_t val)
+int nios_expansion_gpio_dir_write(struct bladerf *dev,
+                                  uint32_t mask, uint32_t val)
 {
     int status = nios_32x32_masked_write(dev, NIOS_PKT_32x32_TARGET_EXP_DIR,
-                                         0xffffffff, val);
+                                         mask, val);
 
     if (status == 0) {
-        log_verbose("%s: Wrote 0x08x\n", __FUNCTION__, val);
+        log_verbose("%s: Wrote 0x%08x (with mask 0x%08x)\n",
+                    __FUNCTION__, val, mask);
     }
 
     return status;
@@ -700,3 +759,79 @@ int nios_retune(struct bladerf *dev, bladerf_module module,
 
     return status;
 }
+
+int nios_read_trigger(struct bladerf *dev, bladerf_module module,
+                      bladerf_trigger_signal trigger, uint8_t *value)
+{
+    int status;
+    uint8_t nios_id;
+
+    switch (module) {
+        case BLADERF_MODULE_TX:
+            nios_id = NIOS_PKT_8x8_TX_TRIGGER_CTL;
+            break;
+
+        case BLADERF_MODULE_RX:
+            nios_id = NIOS_PKT_8x8_RX_TRIGGER_CTL;
+            break;
+
+        default:
+            log_debug("Invalid module: %d\n", module);
+            return BLADERF_ERR_INVAL;
+    }
+
+    /* Only 1 external trigger is currently supported */
+    switch (trigger) {
+        case BLADERF_TRIGGER_J71_4:
+            break;
+
+        default:
+            log_debug("Invalid trigger: %d\n", trigger);
+            return BLADERF_ERR_INVAL;
+    }
+
+    status = nios_8x8_read(dev, nios_id, 0, value);
+    if (status == 0) {
+        log_verbose("%s trigger read value 0x%02x\n", module2str(module), *value);
+    }
+
+    return status;
+}
+
+int nios_write_trigger(struct bladerf *dev, bladerf_module module,
+                       bladerf_trigger_signal trigger, uint8_t value)
+{
+    int status;
+    uint8_t nios_id;
+
+    switch (module) {
+        case BLADERF_MODULE_TX:
+            nios_id = NIOS_PKT_8x8_TX_TRIGGER_CTL;
+            break;
+
+        case BLADERF_MODULE_RX:
+            nios_id = NIOS_PKT_8x8_RX_TRIGGER_CTL;
+            break;
+
+        default:
+            log_debug("Invalid module: %d\n", module);
+            return BLADERF_ERR_INVAL;
+    }
+
+    /* Only 1 external trigger is currently supported */
+    switch (trigger) {
+        case BLADERF_TRIGGER_J71_4:
+            break;
+
+        default:
+            log_debug("Invalid trigger: %d\n", trigger);
+            return BLADERF_ERR_INVAL;
+    }
+
+    status = nios_8x8_write(dev, nios_id, 0, value);
+    if (status == 0) {
+        log_verbose("%s trigger write value 0x%02x\n", module2str(module), value);
+    }
+
+    return status;
+}
