@@ -146,6 +146,29 @@ int nios_vctcxo_trim_dac_write(struct bladerf *dev, uint16_t value);
  */
 int nios_vctcxo_trim_dac_read(struct bladerf *dev, uint16_t *value);
 
+/**
+ * Write VCTCXO tamer mode selection
+ *
+ * @param[in]   dev         Device handle
+ * @param[in]   mode        Tamer mode to use, or BLADERF_VCTCXO_TAMER_DISABLED
+ *                          to disable the tamer.
+ *
+ * @return 0 on success, BLADERF_ERR_* code on error.
+ */
+int nios_set_vctcxo_tamer_mode(struct bladerf *dev,
+                               bladerf_vctcxo_tamer_mode mode);
+
+/**
+ * Read the current VCTCXO mode selection
+ *
+ * @param[in]   dev         Device handle
+ * @param[out]  mode        Current tamer mode in use.
+ *
+ * @return 0 on success, BLADERF_ERR_* code on error.
+ */
+int nios_get_vctcxo_tamer_mode(struct bladerf *dev,
+                               bladerf_vctcxo_tamer_mode *mode);
+
 /**
  * Read a IQ gain correction value
  *
@@ -218,11 +241,12 @@ int nios_expansion_gpio_read(struct bladerf *dev, uint32_t *val);
  * Write to expansion GPIO
  *
  * @param[in]   dev         Device handle
+ * @param[in]   mask        Mask denoting bits to write
  * @param[in]   value       Write value
  *
  * @return 0 on success, BLADERF_ERR_* code on error.
  */
-int nios_expansion_gpio_write(struct bladerf *dev, uint32_t val);
+int nios_expansion_gpio_write(struct bladerf *dev, uint32_t mask, uint32_t val);
 
 /**
  * Read from expansion GPIO direction register
@@ -238,11 +262,13 @@ int nios_expansion_gpio_dir_read(struct bladerf *dev, uint32_t *val);
  * Write to expansion GPIO direction register
  *
  * @param[in]   dev         Device handle
- * @param[in]   value       Write value
+ * @param[in]   mask        Mask denoting bits to configure
+ * @param[in]   output      '1' bit denotes output, '0' bit denotes input
  *
  * @return 0 on success, BLADERF_ERR_* code on error.
  */
-int nios_expansion_gpio_dir_write(struct bladerf *dev, uint32_t val);
+int nios_expansion_gpio_dir_write(struct bladerf *dev,
+                                  uint32_t mask, uint32_t outputs);
 
 /**
  * Dummy handler for a retune request, which is not supported on
@@ -265,4 +291,32 @@ int nios_expansion_gpio_dir_write(struct bladerf *dev, uint32_t val);
 int nios_retune(struct bladerf *dev, bladerf_module module, uint64_t timestamp,
                 uint16_t nint, uint32_t nfrac, uint8_t freqsel, uint8_t vcocap,
                 bool low_band, bool quick_tune);
+
+/**
+ * Read trigger register value
+ *
+ * @param[in]   dev        Device handle
+ * @param[in]   module     Module to configure
+ * @param[in]   trigger    Trigger to read from
+ * @param[out]  value      On success, updated with register value
+ *
+ *
+ * @return 0 on success, BLADERF_ERR_* code on error
+ */
+int nios_read_trigger(struct bladerf *dev, bladerf_module module,
+                      bladerf_trigger_signal trigger, uint8_t * value);
+
+/**
+ * Write trigger register value
+ *
+ * @param[in]   dev        Device handle
+ * @param[in]   module     Module to configure
+ * @param[in]   trigger    Trigger to read
+ * @param[in]   value      Value to write
+ *
+ * @return 0 on success, BLADERF_ERR_* code on error
+ */
+int nios_write_trigger(struct bladerf *dev, bladerf_module module,
+                       bladerf_trigger_signal trigger, uint8_t value);
+
 #endif
