@@ -52,7 +52,7 @@ typedef enum {
 
 static const struct usb_driver *usb_driver_list[] = BLADERF_USB_BACKEND_LIST;
 
-/* FW declaration of fn table declated at the end of this file */
+/* FW declaration of fn table declared at the end of this file */
 const struct backend_fns backend_fns_usb_legacy;
 
 /* Vendor command wrapper to gets a 32-bit integer and supplies a wIndex */
@@ -152,7 +152,7 @@ static int usb_is_fpga_configured(struct bladerf *dev)
 }
 
 /* Switch to RF link mode, fetch the FPGA version, and determine the
- * device's capabiliities */
+ * device's capabilities */
 static int post_fpga_load_init(struct bladerf *dev)
 {
     int status = change_setting(dev, USB_IF_RF_LINK);
@@ -880,11 +880,12 @@ static int usb_stream(struct bladerf_stream *stream, bladerf_module module)
 }
 
 int usb_submit_stream_buffer(struct bladerf_stream *stream, void *buffer,
-                             unsigned int timeout_ms)
+                             unsigned int timeout_ms, bool nonblock)
 {
     void *driver;
     struct bladerf_usb *usb = usb_backend(stream->dev, &driver);
-    return usb->fn->submit_stream_buffer(driver, stream, buffer, timeout_ms);
+    return usb->fn->submit_stream_buffer(driver, stream, buffer,
+                                         timeout_ms, nonblock);
 }
 
 static void usb_deinit_stream(struct bladerf_stream *stream)
@@ -1057,6 +1058,41 @@ static int usb_load_fw_from_bootloader(bladerf_backend backend,
     return status;
 }
 
+/* Default handlers for operations unsupported by the NIOS II legacy packet
+ * format */
+static int set_vctcxo_tamer_mode_unsupported(struct bladerf *dev,
+                                             bladerf_vctcxo_tamer_mode mode)
+{
+    log_debug("Operation not supported with legacy NIOS packet format.\n");
+    return BLADERF_ERR_UNSUPPORTED;
+}
+
+static int get_vctcxo_tamer_mode_unsupported(struct bladerf *dev,
+                                             bladerf_vctcxo_tamer_mode *mode)
+{
+    *mode = BLADERF_VCTCXO_TAMER_INVALID;
+    log_debug("Operation not supported with legacy NIOS packet format.\n");
+    return BLADERF_ERR_UNSUPPORTED;
+}
+
+static int usb_read_fw_log(struct bladerf *dev, logger_entry *e)
+{
+    int status;
+    *e = LOG_EOF;
+
+    if (have_cap(dev, BLADERF_CAP_READ_FW_LOG_ENTRY)) {
+        status = vendor_cmd_int(dev, BLADE_USB_CMD_READ_LOG_ENTRY,
+                                USB_DIR_DEVICE_TO_HOST, (int32_t*) e);
+    } else {
+        log_debug("FX3 FW v%s does not support log retrieval.\n",
+                  dev->fw_version.describe);
+        status = BLADERF_ERR_UNSUPPORTED;
+    }
+
+    return status;
+}
+
+
 /* USB backend that used legacy format for communicating with NIOS II */
 const struct backend_fns backend_fns_usb_legacy = {
     FIELD_INIT(.matches, usb_matches),
@@ -1104,6 +1140,9 @@ const struct backend_fns backend_fns_usb_legacy = {
     FIELD_INIT(.vctcxo_dac_write, nios_legacy_vctcxo_trim_dac_write),
     FIELD_INIT(.vctcxo_dac_read, nios_vctcxo_trim_dac_read),
 
+    FIELD_INIT(.set_vctcxo_tamer_mode, set_vctcxo_tamer_mode_unsupported),
+    FIELD_INIT(.get_vctcxo_tamer_mode, get_vctcxo_tamer_mode_unsupported),
+
     FIELD_INIT(.xb_spi, nios_legacy_xb200_synth_write),
 
     FIELD_INIT(.set_firmware_loopback, usb_set_firmware_loopback),
@@ -1119,6 +1158,8 @@ const struct backend_fns backend_fns_usb_legacy = {
     FIELD_INIT(.retune, nios_retune),
 
     FIELD_INIT(.load_fw_from_bootloader, usb_load_fw_from_bootloader),
+
+    FIELD_INIT(.read_fw_log, usb_read_fw_log),
 };
 
 /* USB backend for use with FPGA supporting update NIOS II packet formats */
@@ -1168,6 +1209,9 @@ const struct backend_fns backend_fns_usb = {
     FIELD_INIT(.vctcxo_dac_write, nios_vctcxo_trim_dac_write),
     FIELD_INIT(.vctcxo_dac_read, nios_vctcxo_trim_dac_read),
 
+    FIELD_INIT(.set_vctcxo_tamer_mode, nios_set_vctcxo_tamer_mode),
+    FIELD_INIT(.get_vctcxo_tamer_mode, nios_get_vctcxo_tamer_mode),
+
     FIELD_INIT(.xb_spi, nios_xb200_synth_write),
 
     FIELD_INIT(.set_firmware_loopback, usb_set_firmware_loopback),
@@ -1183,4 +1227,9 @@ const struct backend_fns backend_fns_usb = {
     FIELD_INIT(.retune, nios_retune),
 
     FIELD_INIT(.load_fw_from_bootloader, usb_load_fw_from_bootloader),
+
+    FIELD_INIT(.read_fw_log, usb_read_fw_log),
+
+    FIELD_INIT(.read_trigger, nios_read_trigger),
+    FIELD_INIT(.write_trigger, nios_write_trigger),
 };
