@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Nuand LLC
+ * Copyright (C) 2014-2015 Nuand LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,6 +71,11 @@ int sync_init(struct bladerf *dev,
         return BLADERF_ERR_INVAL;
     }
 
+    if (module != BLADERF_MODULE_TX && module != BLADERF_MODULE_RX) {
+        log_debug("Invalid bladerf_module value encountered: %d", module);
+        return BLADERF_ERR_INVAL;
+    }
+
     switch (format) {
         case BLADERF_FORMAT_SC16_Q11:
         case BLADERF_FORMAT_SC16_Q11_META:
@@ -88,25 +93,27 @@ int sync_init(struct bladerf *dev,
     }
 
     /* Deallocate any existing sync handle for this module */
+    sync_deinit(dev->sync[module]);
+
+    sync = dev->sync[module] =
+        (struct bladerf_sync *) calloc(1, sizeof(struct bladerf_sync));
+
+    if (dev->sync[module] == NULL) {
+        return BLADERF_ERR_MEM;
+    }
+
     switch (module) {
         case BLADERF_MODULE_TX:
-        case BLADERF_MODULE_RX:
-            sync_deinit(dev->sync[module]);
-            sync = dev->sync[module] =
-                (struct bladerf_sync *) calloc(1, sizeof(struct bladerf_sync));
+            sync->buf_mgmt.submitter = SYNC_TX_SUBMITTER_FN;
+            break;
 
-            if (dev->sync[module] == NULL) {
-                status = BLADERF_ERR_MEM;
-            }
+        case BLADERF_MODULE_RX:
+            sync->buf_mgmt.submitter = SYNC_TX_SUBMITTER_INVALID;
             break;
 
         default:
-            log_debug("Invalid bladerf_module value encountered: %d", module);
-            status = BLADERF_ERR_INVAL;
-    }
-
-    if (status != 0) {
-        return status;
+            log_debug("Invalid module provided: %d\n", module);
+            return BLADERF_ERR_INVAL;
     }
 
     sync->dev = dev;
@@ -165,7 +172,7 @@ int sync_init(struct bladerf *dev,
 
             case BLADERF_MODULE_TX:
                 sync->buf_mgmt.prod_i = 0;
-                sync->buf_mgmt.cons_i = 0;
+                sync->buf_mgmt.cons_i = BUFFER_MGMT_INVALID_INDEX;
                 sync->buf_mgmt.partial_off = 0;
 
                 for (i = 0; i < num_buffers; i++) {
@@ -174,8 +181,10 @@ int sync_init(struct bladerf *dev,
 
                 sync->meta.in_burst = false;
                 sync->meta.now = false;
-
                 break;
+
+            default:
+                return BLADERF_ERR_INVAL;
         }
 
         status = sync_worker_init(sync);
@@ -195,7 +204,7 @@ void sync_deinit(struct bladerf_sync *sync)
 
         if (sync->stream_config.module == BLADERF_MODULE_TX) {
             async_submit_stream_buffer(sync->worker->stream,
-                                       BLADERF_STREAM_SHUTDOWN, 0);
+                                       BLADERF_STREAM_SHUTDOWN, 0, false);
         }
 
         sync_worker_deinit(sync->worker, &sync->buf_mgmt.lock,
@@ -214,10 +223,12 @@ static int wait_for_buffer(struct buffer_mgmt *b, unsigned int timeout_ms,
     struct timespec timeout;
 
     if (timeout_ms == 0) {
-        log_verbose("%s: Infinite wait for [%d] to fill.\n", dbg_name, dbg_idx);
+        log_verbose("%s: Infinite wait for buffer[%d] (status: %d).\n",
+                    dbg_name, dbg_idx, b->status[dbg_idx]);
         status = pthread_cond_wait(&b->buf_ready, &b->lock);
     } else {
-        log_verbose("%s: Timed wait for [%d] to fill.\n", dbg_name, dbg_idx);
+        log_verbose("%s: Timed wait for buffer[%d] (status: %d).\n",
+                    dbg_name, dbg_idx, b->status[dbg_idx]);
         status = populate_abs_timeout(&timeout, timeout_ms);
         if (status == 0) {
             status = pthread_cond_timedwait(&b->buf_ready, &b->lock, &timeout);
@@ -385,7 +396,6 @@ int sync_rx(struct bladerf *dev, void *samples, unsigned num_samples,
                 MUTEX_LOCK(&b->lock);
                 b->status[b->cons_i] = SYNC_BUFFER_PARTIAL;
                 b->partial_off = 0;
-                MUTEX_UNLOCK(&b->lock);
 
                 switch (s->stream_config.format) {
                     case BLADERF_FORMAT_SC16_Q11:
@@ -402,6 +412,8 @@ int sync_rx(struct bladerf *dev, void *samples, unsigned num_samples,
                         assert(!"Invalid stream format");
                         status = BLADERF_ERR_UNEXPECTED;
                 }
+
+                MUTEX_UNLOCK(&b->lock);
                 break;
 
             case SYNC_STATE_USING_BUFFER: /* SC16Q11 buffers w/o metadata */
@@ -610,36 +622,73 @@ int sync_rx(struct bladerf *dev, void *samples, unsigned num_samples,
 /* Assumes buffer lock is held */
 static int advance_tx_buffer(struct bladerf_sync *s, struct buffer_mgmt *b)
 {
-    int status;
+    int status = 0;
+    const unsigned int idx = b->prod_i;
 
-    log_verbose("%s: Marking buf[%u] full\n", __FUNCTION__, b->prod_i);
-    b->status[b->prod_i] = SYNC_BUFFER_IN_FLIGHT;
+    if (b->submitter == SYNC_TX_SUBMITTER_FN) {
+        /* Mark buffer in flight because we're going to send it out.
+         * This ensures that if the callback fires before this function
+         * completes, its state will be correct. */
+        b->status[idx] = SYNC_BUFFER_IN_FLIGHT;
 
-    /* This call may block and it results in a per-stream lock being held, so
-     * the buffer lock must be dropped.
-     *
-     * A callback may occur in the meantime, but this will not touch the status
-     * for this this buffer, or the producer index.
-     */
-    MUTEX_UNLOCK(&b->lock);
-    status = async_submit_stream_buffer(s->worker->stream,
-                                        b->buffers[b->prod_i],
-                                        s->stream_config.timeout_ms);
-    MUTEX_LOCK(&b->lock);
+        /* This call may block and it results in a per-stream lock being held,
+         * so the buffer lock must be dropped.
+         *
+         * A callback may occur in the meantime, but this will not touch the
+         * status for this this buffer, or the producer index.
+         */
+        MUTEX_UNLOCK(&b->lock);
+        status = async_submit_stream_buffer(s->worker->stream,
+                                            b->buffers[idx],
+                                            s->stream_config.timeout_ms,
+                                            true);
+        MUTEX_LOCK(&b->lock);
 
-    if (status == 0) {
-        b->prod_i = (b->prod_i + 1) % b->num_buffers;
+        if (status == 0) {
+            log_verbose("%s: buf[%u] submitted.\n",
+                        __FUNCTION__, idx);
 
-        /* Go handle the next buffer, if we have one available.  Otherwise,
-         * check up on the worker's state and restart it if needed. */
-        if (b->status[b->prod_i] == SYNC_BUFFER_EMPTY) {
-            s->state = SYNC_STATE_BUFFER_READY;
+        } else if (status == BLADERF_ERR_WOULD_BLOCK) {
+            log_verbose("%s: Deferring buf[%u] submission to worker callback.\n",
+                        __FUNCTION__, idx);
+
+            /* Mark this buffer as being full of data, but not in flight */
+            b->status[idx] = SYNC_BUFFER_FULL;
+
+            /* Assign callback the duty of submitting deferred buffers,
+             * and use buffer_mgmt.cons_i to denote which it should submit
+             * (i.e., consume). */
+            b->submitter = SYNC_TX_SUBMITTER_CALLBACK;
+            b->cons_i = idx;
+
+            /* This is expected and we are handling it. Don't propagate this
+             * status back up */
+            status = 0;
         } else {
-            s->state = SYNC_STATE_CHECK_WORKER;
-        }
+            /* Unmark this as being in flight */
+            b->status[idx] = SYNC_BUFFER_FULL;
+
+            log_debug("%s: Failed to submit buf[%u].\n", __FUNCTION__, idx);
+            return status;
+       }
     } else {
-        log_debug("%s: Failed to advance buffer: %s\n",
-                  __FUNCTION__, bladerf_strerror(status));
+        /* We are not submitting this buffer; this is deffered to the worker
+         * call back. Just update its state to being full of samples. */
+        b->status[idx] = SYNC_BUFFER_FULL;
+    }
+
+    /* Advance "producer" insertion index. */
+    b->prod_i = (idx + 1) % b->num_buffers;
+
+    /* Determine our next state based upon the state of the next buffer we
+     * want to use. */
+    if (b->status[b->prod_i] == SYNC_BUFFER_EMPTY) {
+        /* Buffer is empty and ready for use */
+        s->state = SYNC_STATE_BUFFER_READY;
+    } else {
+        /* We'll have to wait on this buffer to become ready. First, we'll
+         * verify that the worker is running. */
+        s->state = SYNC_STATE_CHECK_WORKER;
     }
 
     return status;
@@ -769,11 +818,15 @@ int sync_tx(struct bladerf *dev, void *samples, unsigned int num_samples,
                     status = stream_error;
                 } else {
                     if (worker_state == SYNC_WORKER_STATE_IDLE) {
-                        /* No need to reset any buffer managment for TX since
+                        /* No need to reset any buffer management for TX since
                          * the TX stream does not submit an initial set of
                          * buffers.  Therefore the RESET_BUF_MGMT state is
                          * skipped here. */
                         s->state = SYNC_STATE_START_WORKER;
+                    } else {
+                        /* Worker is running - continue onto checking for and
+                         * potentially waiting for an available buffer */
+                        s->state = SYNC_STATE_WAIT_FOR_BUFFER;
                     }
                 }
                 break;
@@ -816,7 +869,6 @@ int sync_tx(struct bladerf *dev, void *samples, unsigned int num_samples,
                 MUTEX_LOCK(&b->lock);
                 b->status[b->prod_i] = SYNC_BUFFER_PARTIAL;
                 b->partial_off = 0;
-                MUTEX_UNLOCK(&b->lock);
 
                 switch (s->stream_config.format) {
                     case BLADERF_FORMAT_SC16_Q11:
@@ -833,6 +885,8 @@ int sync_tx(struct bladerf *dev, void *samples, unsigned int num_samples,
                         assert(!"Invalid stream format");
                         status = BLADERF_ERR_UNEXPECTED;
                 }
+
+                MUTEX_UNLOCK(&b->lock);
                 break;
 
 
@@ -1040,8 +1094,10 @@ int sync_tx(struct bladerf *dev, void *samples, unsigned int num_samples,
         }
     }
 
-    if (s->stream_config.format == BLADERF_FORMAT_SC16_Q11_META &&
+    if (status == 0 &&
+        s->stream_config.format == BLADERF_FORMAT_SC16_Q11_META &&
         (user_meta->flags & BLADERF_META_FLAG_TX_BURST_END)) {
+
         s->meta.in_burst = false;
         s->meta.now = false;
     }
