@@ -32,6 +32,7 @@
 #include "lms.h"
 #include "rel_assert.h"
 #include "log.h"
+#include "tuning.h"
 
 #define BLADERF_CONFIG_RX_SWAP_IQ 0x20000000
 #define BLADERF_CONFIG_TX_SWAP_IQ 0x10000000
@@ -61,9 +62,23 @@
 #define BLADERF_XB_RX_MASK   0x30000000
 #define BLADERF_XB_RX_SHIFT  28
 
+#define BLADERF_XB_AUX_EN   0x000002
+#define BLADERF_XB_TX_LED   0x000010
+#define BLADERF_XB_RX_LED   0x000020
+#define BLADERF_XB_TRX_TXn  0x000040
+#define BLADERF_XB_TRX_RXn  0x000080
+#define BLADERF_XB_TRX_MASK 0x0000c0
+#define BLADERF_XB_PA_EN    0x000200
+#define BLADERF_XB_LNA_ENn  0x000400
+#define BLADERF_XB_CS       0x010000
+#define BLADERF_XB_CSEL     0x040000
+#define BLADERF_XB_DOUT     0x100000
+#define BLADERF_XB_SCLK     0x400000
+
 static int xb200_attach(struct bladerf *dev) {
     int status = 0;
     uint32_t val;
+    uint8_t val8;
     unsigned int muxout = 6;
     const char *mux_lut[] = {
         "THREE-STATE OUTPUT",
@@ -78,7 +93,12 @@ static int xb200_attach(struct bladerf *dev) {
 
 
     log_debug("  Attaching transverter board\n");
-    if ((status = SI5338_WRITE(dev, 39, 2)))
+    status = SI5338_READ(dev, 39, &val8);
+    if (status < 0) {
+        return status;
+    }
+    val8 |= 2;
+    if ((status = SI5338_WRITE(dev, 39, val8)))
         return status;
     if ((status = SI5338_WRITE(dev, 34, 0x22)))
         return status;
@@ -90,10 +110,10 @@ static int xb200_attach(struct bladerf *dev) {
     if ((status = XB_GPIO_READ(dev, &val)))
         return status;
 
-    if ((status = XB_GPIO_DIR_WRITE(dev, 0x3C00383E)))
+    if ((status = XB_GPIO_DIR_WRITE(dev, 0xffffffff, 0x3C00383E)))
         return status;
 
-    if ((status = XB_GPIO_WRITE(dev, 0x800)))
+    if ((status = XB_GPIO_WRITE(dev, 0xffffffff, 0x800)))
         return status;
 
     // Load ADF4351 registers via SPI
@@ -121,7 +141,7 @@ static int xb200_attach(struct bladerf *dev) {
     else {
         log_debug("  MUXOUT Bit not set: FAIL\n");
     }
-    status = XB_GPIO_WRITE(dev, 0x3C000800);
+    status = XB_GPIO_WRITE(dev, 0xffffffff, 0x3C000800);
 
     return status;
 }
@@ -143,7 +163,39 @@ int xb200_enable(struct bladerf *dev, bool enable) {
     if (status || (val == orig))
         return status;
 
-    return XB_GPIO_WRITE(dev, val);
+    return XB_GPIO_WRITE(dev, 0xffffffff, val);
+}
+
+static int xb300_attach(struct bladerf *dev) {
+    int status;
+    uint32_t val;
+
+    val = BLADERF_XB_TX_LED | BLADERF_XB_RX_LED | BLADERF_XB_TRX_MASK;
+    val |= BLADERF_XB_PA_EN | BLADERF_XB_LNA_ENn;
+    val |= BLADERF_XB_CSEL  | BLADERF_XB_SCLK | BLADERF_XB_CS;
+
+    if ((status = XB_GPIO_DIR_WRITE(dev, 0xffffffff, val)))
+        return status;
+
+    val = BLADERF_XB_CS | BLADERF_XB_LNA_ENn;
+    if ((status = XB_GPIO_WRITE(dev, 0xffffffff, val)))
+        return status;
+
+    return status;
+}
+
+static int xb300_enable(struct bladerf *dev, bool enable) {
+    int status;
+    uint32_t val;
+    float pwr;
+
+    val = BLADERF_XB_CS | BLADERF_XB_CSEL | BLADERF_XB_LNA_ENn;
+    if ((status = XB_GPIO_WRITE(dev, 0xffffffff, val)))
+        return status;
+
+    status = xb300_get_output_power(dev, &pwr);
+
+    return status;
 }
 
 int xb_attach(struct bladerf *dev, bladerf_xb xb) {
@@ -163,15 +215,39 @@ int xb_attach(struct bladerf *dev, bladerf_xb xb) {
     switch (xb) {
         /* This requires support from the FPGA and FX3 firmware to be added */
         case BLADERF_XB_100:
-            log_debug("%s: The XB-100 is not currently supported by "
-                      "libbladeRF\n", __FUNCTION__);
-            status = BLADERF_ERR_UNSUPPORTED;
+            if (!have_cap(dev, BLADERF_CAP_MASKED_XBIO_WRITE)) {
+                log_debug("%s: XB100 support requires FPGA v0.4.1 or later.\n",
+                          __FUNCTION__);
+                return BLADERF_ERR_UNSUPPORTED;
+            } else {
+                const uint32_t mask =
+                    BLADERF_XB100_LED_D1        |
+                    BLADERF_XB100_LED_D2        |
+                    BLADERF_XB100_LED_D3        |
+                    BLADERF_XB100_LED_D4        |
+                    BLADERF_XB100_LED_D5        |
+                    BLADERF_XB100_LED_D6        |
+                    BLADERF_XB100_LED_D7        |
+                    BLADERF_XB100_LED_D8        |
+                    BLADERF_XB100_TLED_RED      |
+                    BLADERF_XB100_TLED_GREEN    |
+                    BLADERF_XB100_TLED_BLUE;
+
+                const uint32_t outputs = mask;
+                const uint32_t default_values = mask;
+
+                status = XB_GPIO_DIR_WRITE(dev, mask, outputs);
+                if (status == 0) {
+                    status = XB_GPIO_WRITE(dev, mask, default_values);
+                }
+            }
+
             break;
 
         case BLADERF_XB_200:
             if (!have_cap(dev, BLADERF_CAP_XB200)) {
-                log_warning("%s: XB200 support requires FPGA v0.0.5 or later\n",
-                            __FUNCTION__);
+                log_debug("%s: XB200 support requires FPGA v0.0.5 or later\n",
+                          __FUNCTION__);
                 status = BLADERF_ERR_UPDATE_FPGA;
             } else if (attached != xb) {
                 log_verbose( "Attaching XB200\n" );
@@ -192,6 +268,36 @@ int xb_attach(struct bladerf *dev, bladerf_xb xb) {
                 log_verbose( "Setting TX path\n" );
                 status = xb200_set_path(dev, BLADERF_MODULE_TX, BLADERF_XB200_BYPASS);
             }
+            if (status != 0) {
+                break;
+            }
+            log_verbose( "Setting RX filter\n" );
+            status = xb200_set_filterbank(dev, BLADERF_MODULE_RX, BLADERF_XB200_AUTO_1DB);
+            if (status != 0) {
+                break;
+            }
+            log_verbose( "Setting TX filter\n" );
+            status = xb200_set_filterbank(dev, BLADERF_MODULE_TX, BLADERF_XB200_AUTO_1DB);
+         break;
+
+        case BLADERF_XB_300:
+            if (attached != xb) {
+                log_verbose( "Attaching XB\n" );
+                status = xb300_attach(dev);
+                if (status != 0) {
+                    break;
+                }
+                log_verbose( "Enabling XB\n" );
+                status = xb300_enable(dev, true);
+                if (status != 0) {
+                    break;
+                }
+                log_verbose( "Setting TRX path to TX\n" );
+                status = xb300_set_trx(dev, BLADERF_XB300_TRX_TX);
+                if (status != 0) {
+                    break;
+                }
+            }
             break;
 
         case BLADERF_XB_NONE:
@@ -260,22 +366,15 @@ int xb200_get_filterbank(struct bladerf *dev, bladerf_module module,
     return status;
 }
 
-static int set_filterbank(struct bladerf *dev, bool update_auto_filt,
-                          bladerf_module module, bladerf_xb200_filter filter)
+static int set_filterbank_mux(struct bladerf *dev, bladerf_module module, bladerf_xb200_filter filter)
 {
     int status;
     uint32_t orig, val, mask;
     unsigned int shift;
+    static const char *filters[] = { "50M", "144M", "222M", "custom" };
 
-    status = check_module(module);
-    if (status != 0) {
-        return status;
-    }
-
-    status = check_xb200_filter(filter);
-    if (status != 0) {
-        return status;
-    }
+    assert(filter >= 0);
+    assert(filter < ARRAY_SIZE(filters));
 
     if (module == BLADERF_MODULE_RX) {
         mask = BLADERF_XB_RX_MASK;
@@ -294,36 +393,51 @@ static int set_filterbank(struct bladerf *dev, bool update_auto_filt,
     val |= filter << shift;
 
     if (orig != val) {
-        status = XB_GPIO_WRITE(dev, val);
+        log_debug("Engaging %s band XB-200 %s filter\n", filters[filter],
+            mask == BLADERF_XB_TX_MASK ? "TX" : "RX");
+
+        status = XB_GPIO_WRITE(dev, 0xffffffff, val);
         if (status != 0) {
             return status;
         }
     }
 
 
-    /*
-     * Update state information regarding automatically selected filter.
-     * Invalidate the entry if we're not entering automatic selection mode.
-     *
-     * We'll suppress this update when we know we're calling this function
-     * from xb200_auto_filter_selection for an automatic update so that we
-     * don't clobber our auto_filter mode.
-     */
-    if (update_auto_filt) {
-        if (filter == BLADERF_XB200_AUTO_1DB || filter == BLADERF_XB200_AUTO_3DB) {
-            dev->auto_filter[module] = filter;
-        } else {
-            dev->auto_filter[module] = -1;
-        }
-    }
-
     return 0;
 }
 
 int xb200_set_filterbank(struct bladerf *dev,
                          bladerf_module module, bladerf_xb200_filter filter) {
 
-    return set_filterbank(dev, true, module, filter);
+    int status = 0;
+    unsigned int frequency;
+
+    status = check_module(module);
+    if (status != 0) {
+        return status;
+    }
+
+    status = check_xb200_filter(filter);
+    if (status != 0) {
+        return status;
+    }
+
+    if (filter == BLADERF_XB200_AUTO_1DB || filter == BLADERF_XB200_AUTO_3DB) {
+        /* Save which soft auto filter mode we're in */
+        dev->auto_filter[module] = filter;
+
+        status = tuning_get_freq(dev, module, &frequency);
+        if (status == 0) {
+            status = xb200_auto_filter_selection(dev, module, frequency);
+        }
+
+    } else {
+        /* Invalidate the soft auto filter mode entry */
+        dev->auto_filter[module] = -1;
+        status = set_filterbank_mux(dev, module, filter);
+    }
+
+    return status;
 }
 
 int xb200_auto_filter_selection(struct bladerf *dev, bladerf_module mod,
@@ -351,7 +465,7 @@ int xb200_auto_filter_selection(struct bladerf *dev, bladerf_module mod,
             filter = BLADERF_XB200_CUSTOM;
         }
 
-        status = set_filterbank(dev, false, mod, filter);
+        status = set_filterbank_mux(dev, mod, filter);
     } else if (dev->auto_filter[mod] == BLADERF_XB200_AUTO_3DB) {
         if (34782924 <= frequency && frequency <= 61899260) {
             filter = BLADERF_XB200_50M;
@@ -363,7 +477,7 @@ int xb200_auto_filter_selection(struct bladerf *dev, bladerf_module mod,
             filter = BLADERF_XB200_CUSTOM;
         }
 
-        status = set_filterbank(dev, false, mod, filter);
+        status = set_filterbank_mux(dev, mod, filter);
     }
 
     return status;
@@ -448,7 +562,7 @@ int xb200_set_path(struct bladerf *dev,
         }
     }
 
-    return XB_GPIO_WRITE(dev, val);
+    return XB_GPIO_WRITE(dev, 0xffffffff, val);
 }
 
 int xb200_get_path(struct bladerf *dev,
@@ -478,3 +592,214 @@ int xb200_get_path(struct bladerf *dev,
     return 0;
 }
 
+int xb300_set_trx(struct bladerf *dev, bladerf_xb300_trx trx)
+{
+    int status;
+    uint32_t val;
+
+    status = XB_GPIO_READ(dev, &val);
+    if (status != 0) {
+        return status;
+    }
+
+    val &= ~(BLADERF_XB_TRX_MASK);
+
+    switch (trx) {
+        case BLADERF_XB300_TRX_RX:
+            val |= BLADERF_XB_TRX_RXn;
+            break;
+
+        case BLADERF_XB300_TRX_TX:
+            val |= BLADERF_XB_TRX_TXn;
+            break;
+
+        case BLADERF_XB300_TRX_UNSET:
+            break;
+
+        default:
+            log_debug("Invalid TRX option: %d\n", trx);
+            return BLADERF_ERR_INVAL;
+    }
+
+    status = XB_GPIO_WRITE(dev, 0xffffffff, val);
+    return status;
+}
+
+int xb300_get_trx(struct bladerf *dev, bladerf_xb300_trx *trx)
+{
+    int status;
+    uint32_t val;
+
+    *trx = BLADERF_XB300_TRX_INVAL;
+
+    status = XB_GPIO_READ(dev, &val);
+    if (status != 0) {
+        return status;
+    }
+
+    val &= BLADERF_XB_TRX_MASK;
+
+    if (!val) {
+        *trx = BLADERF_XB300_TRX_UNSET;
+    } else {
+        *trx = (val & BLADERF_XB_TRX_RXn) ? BLADERF_XB300_TRX_RX : BLADERF_XB300_TRX_TX;
+    }
+
+    /* Sanity check */
+    switch (*trx) {
+        case BLADERF_XB300_TRX_TX:
+        case BLADERF_XB300_TRX_RX:
+        case BLADERF_XB300_TRX_UNSET:
+            break;
+
+        default:
+            log_debug("Read back invalid TRX setting value: %d\n", *trx);
+            status = BLADERF_ERR_INVAL;
+    }
+
+    return status;
+}
+
+int xb300_set_amplifier_enable(struct bladerf *dev,
+                               bladerf_xb300_amplifier amp, bool enable) {
+    int status;
+    uint32_t val;
+
+    status = XB_GPIO_READ(dev, &val);
+    if (status != 0) {
+        return status;
+    }
+
+    switch (amp) {
+        case BLADERF_XB300_AMP_PA:
+            if (enable) {
+                val |= BLADERF_XB_TX_LED;
+                val |= BLADERF_XB_PA_EN;
+            } else {
+                val &= ~BLADERF_XB_TX_LED;
+                val &= ~BLADERF_XB_PA_EN;
+            }
+            break;
+
+        case BLADERF_XB300_AMP_LNA:
+            if (enable) {
+                val |= BLADERF_XB_RX_LED;
+                val &= ~BLADERF_XB_LNA_ENn;
+            } else {
+                val &= ~BLADERF_XB_RX_LED;
+                val |= BLADERF_XB_LNA_ENn;
+            }
+            break;
+
+        case BLADERF_XB300_AMP_PA_AUX:
+            if (enable) {
+                val |= BLADERF_XB_AUX_EN;
+            } else {
+                val &= ~BLADERF_XB_AUX_EN;
+            }
+            break;
+
+        default:
+            log_debug("Invalid amplifier selection: %d\n", amp);
+            return BLADERF_ERR_INVAL;
+    }
+
+    status = XB_GPIO_WRITE(dev, 0xffffffff, val);
+
+    return status;
+}
+
+int xb300_get_amplifier_enable(struct bladerf *dev,
+                               bladerf_xb300_amplifier amp, bool *enable)
+{
+    int status;
+    uint32_t val;
+
+    *enable = false;
+
+    status = XB_GPIO_READ(dev, &val);
+    if (status != 0) {
+        return status;
+    }
+
+    switch (amp) {
+        case BLADERF_XB300_AMP_PA:
+            *enable = (val & BLADERF_XB_PA_EN);
+            break;
+
+        case BLADERF_XB300_AMP_LNA:
+            *enable = !(val & BLADERF_XB_LNA_ENn);
+            break;
+
+        case BLADERF_XB300_AMP_PA_AUX:
+            *enable = (val & BLADERF_XB_AUX_EN);
+            break;
+
+        default:
+            log_debug("Read back invalid amplifier setting: %d\n", amp);
+            status = BLADERF_ERR_INVAL;
+    }
+
+    return status;
+}
+
+int xb300_get_output_power(struct bladerf *dev, float *pwr)
+{
+    uint32_t val, rval;
+    int i;
+    int ret = 0;
+    int status;
+    float volt, volt2, volt3, volt4;
+
+    status = XB_GPIO_READ(dev, &val);
+    if (status) {
+        return status;
+    }
+
+    val &= ~(BLADERF_XB_CS | BLADERF_XB_SCLK | BLADERF_XB_CSEL);
+
+    status = XB_GPIO_WRITE(dev, 0xffffffff, BLADERF_XB_SCLK | val);
+    if (status) {
+        return status;
+    }
+
+    status = XB_GPIO_WRITE(dev, 0xffffffff, BLADERF_XB_CS | BLADERF_XB_SCLK | val);
+    if (status) {
+        return status;
+    }
+
+    for (i = 1; i <= 14; i++) {
+        status = XB_GPIO_WRITE(dev, 0xffffffff, val);
+        if (status) {
+            return status;
+        }
+
+        status = XB_GPIO_WRITE(dev, 0xffffffff, BLADERF_XB_SCLK | val);
+        if (status) {
+            return status;
+        }
+
+        status = XB_GPIO_READ(dev, &rval);
+        if (status) {
+            return status;
+        }
+
+        if (i >= 2 && i <= 11) {
+            ret |= (!!(rval & BLADERF_XB_DOUT)) << (11 - i);
+        }
+    }
+
+    volt = (1.8f/1024.0f) * ret;
+    volt2 = volt  * volt;
+    volt3 = volt2 * volt;
+    volt4 = volt3 * volt;
+
+    *pwr =  -503.933f  * volt4 +
+            1409.489f  * volt3 -
+            1487.84f   * volt2 +
+             722.9793f * volt  -
+             114.7529f;
+
+    return 0;
+
+}
